#!/usr/bin/perl
# batch-delete-messages.pl - delete a bunch of messages.
# Copyright HyperNews.org, 1998

# Written by Daniel LaLiberte for HyperNews.org.

###########################################################

# This script deletes messages in a forum whose top-level thread message is 
# older than a set number of days.  It adjusts the number of messages
# that the forum contains.

# Default maximum age, in days.
my $max_age = 30;

# The max age can be overridden with the -T or --age command line option.
# The forum is named with the -F, --forum, or --dir command line option.
# Try `batch-delete-messages.pl --help` for a list of more options.

# Here is an example of a crontab entry that runs at 4:17 am every day
# and deletes messages older than 90 days.
# 17 4 * * * nice $HOME/httpd/cgi-bin/HyperNews/.scripts/batch-delete-messages.pl --dir /forum --age 90

# Other options that should be added: not available yet.  They might include:
#  * Delete threads whose newest message is older that some age. (easy)
#  * Delete only half or up to a max number of deletable messages. (easy)
#  * Move old messages to another "archive" forum. (medium hard)
#  * Apply this over several forums or subforums. (fairly easy)
#  * Delete messages with relatively few readers. (how much is few?)
#  * Delete expired messages.  (no 'expiration' currently)
#  * Delete expired placeholder messages. (easy)

###########################################################

# We have to do some things with @INC in order for this to compile right.
BEGIN 
{
    # Prepend HyperNews directories to load path.
    $0 =~ m{^(.*)/[^/]+$ }x;   $hnbin = $1 || '.';
    $hnbin =~ s{/(.scripts|Admin|SECURED)}{}g;
    unshift(@INC, $hnbin, "$hnbin/.scripts"); 
}

require "Admin/hnrc";

 
local $plaintext = 1;

$debug = 1;

####################################
# Option variables

$forum = '';
$check_only = 0;
$inactive = 0;

####################################

&feval('&BatchMainPL()');


sub BatchMainPL {

    require "urc.pl";
    require 'wwwdates.pl';
    require 'response-lib.pl';
    require 'add-lib.pl';

  push (@INC, "$hnbin/.scripts/lib");
  require Getopt::Declare; import Getopt::Declare;


    # Command line args:
    #  [required] doesnt work with [ditto]
    # --verbose		Verbose mode - doesnt work with --version .. sometimes
    # { $::debug = 1; }

    ##################################################################
    my $optctl = '
        --forum <directory>	Name of forum to operate on.
				Relative to HyperNews document root.
        { $::forum = $directory; }
        --dir <directory>	[ditto]
        -F <directory>		[ditto]

        --age <age>		Maximum age, in days, of messages.
				Default is 30 days.
       { $::max_age = $age; }
        -T <age>		[ditto]

	--inactive		Operate on the messages whose newest
	    			reply (or other change) is older than the age.
       { $::inactive = 1; }

	--check			Check only - don\'t change anything.
       { $::check_only = 1; }

	--quiet			No output.
       { $::debug = 0; }

 	--verbose		Opposite of quiet.
       { $::debug = 1; }
';

    ##################################################################

    $args = new Getopt::Declare($optctl);

    print ("Checking only...\n") if $debug && $check_only;

    print "\n" if $debug;

    # Add leading / and delete trailing / or .html, if needed.
    $forum =~ s{^([^/])} {/$1};
    $forum =~ s{/$ } {}x;
    $forum =~ s{\.html$ } {}x;


    &HNError("You must specify a forum to operate on.") if (!$forum);

    if ($debug) {
      my $old_or_inactive = ($inactive 
			     ? "have been inactive for $max_age days"
			     : "were started more than $max_age days ago");
	print qq{
Deleting threads from forum '$forum'
that $old_or_inactive.

};
    }


#    print ("Opendir $hndocs$forum\n") if $debug;
    opendir (DIR, "$hndocs$forum") ||
	&HNError("Problem reading '$hndocs$forum'.");

    my @files = readdir DIR;
    closedir DIR;

    # Look at the message urc files.
    @files = grep (/^[0-9]+\.html,urc$/, @files);

    # &HNMsg ("Files: @files\n") if $debug;

    my $today = time;
    my $max_age_in_seconds = $max_age * 24 * 60 * 60;

    # Accumulate old files here first.
    my @oldfiles = ();
    my $delete_num = 0;
    my $total_num = 0;

    my $which_date = 'Date';  # creation date
    $which_date = 'LastMod' if $inactive;  # last reply or modification.

    foreach $file (@files) {
	if ($file !~ m|(.*)\.html,urc$|) {
	    next;
	}
	my $root = $1;

	my %urc = &parseURCFile("$hndocs$forum/$root.html,urc");
	next if !%urc; # aint nothing here.
	
	my $num_here = $urc{NumMessages} || 1;  # is "|| 1" correct?
	$total_num += $num_here;

	my $date = $urc{$which_date};
        my $time = &wwwdates::get_gmtime($date);

	if (($today - $time) > $max_age_in_seconds) {
	  $delete_num += $num_here;
	  print "$date - too old: $root: $num_here message(s)\n" 
	    if $debug;
	  push @oldfiles, $root;
	}
    }


    print "\nDeleting $delete_num of $total_num messages.\n" if $debug;

    print ("Checking only ... done!\n") if $debug && $check_only;

    return if $check_only;

    @oldfiles = sort @oldfiles;  # Sort by message num, date added.
    my $num_files = @oldfiles;

    # Now remove them.
    require 'lock.pl';
    foreach $root (@oldfiles) {
	&lock_tree("$hndocs$forum/$root");
	print "Deleting: $root\n" if $debug;
	&rmdirR("$hndocs$forum/$root");
	unlink "$hndocs$forum/$root-body.html";
	unlink "$hndocs$forum/$root.html,urc";
	&unlock("$hndocs$forum/$root");
    }

    # Do this after unlocking - this locks also.
    # &clearResponseList($forum);
    &updateLists($forum, $forum, -$num_deleted);


    ########################
    # Clean up the date dirs
    # This should be part of the removing process.

    my $byDateDir = "$hndocs$forum/.byDate";
    return if (!-d $byDateDir);
    opendir (DIR, $byDateDir) ||
	&HNError("Can't open directory $byDateDir");
    my @dirs = grep (/^[^.]/, readdir DIR);
    closedir DIR;

    foreach $dir (@dirs) {
	&HNMsg("Cleaning: $dir") if $debug;
	opendir (DIR, "$byDateDir/$dir");
	my @files = readdir DIR;
	@files = grep (/^[^.]/, @files);
	my $numFiles = @files;
	closedir DIR;
	foreach $link (@files) {
	    my $path = "$byDateDir/$dir/$link";
	    if (!stat $path) {
		$numFiles--;
		unlink $path;
		&HNMsg("Unlinked: $link") if $debug;
	    }
	}
	if ($numFiles <= 0) {
	    # Nothing left here, so unlink it.
	    rmdir "$byDateDir/$dir";
	    &HNMsg("Unlinked: $dir") if $debug;
	}
    }


}


sub rmdirR {
    my ($Dir) = @_;
    local $debug = 0;
    # Recursively remove a directory and contents == rm -r $Dir
    print ("Deleting $Dir\n") if $debug; 
    # Open the directory to read the files
    opendir (DIR, "$Dir");
    my @files = readdir DIR;
    closedir DIR;

    my $file;
    foreach $file (@files) {
	print ("File is \"$file\"\n") if $debug;
	if (($file ne ".") && ($file ne "..")) {
	    $file = &untaintPath($file);
	    if (-d "$Dir/$file") {
		print ("Is Directory...\n") if $debug;
		&rmdirR("$Dir/$file"); 
	    } else {
		unlink("$Dir/$file");
	    }
	}
    }
    rmdir("$Dir");
}
