# delete-lib.pl - HyperNews delete library
# Copyright Board of Trustees of the University of Illinois at UC, April 1995

require "urc.pl";
require "wwwdates.pl";
require "lock.pl";
require "response-lib.pl";
require 'log-lib.pl';


sub deleteTree {
    my ($url, $expires, $forward) = @_;
    # Remove a tree of responses under $url
    # It will leave a placeholder if $expires is given.
    # $expires is the relative number of days after which
    # the placeholder will be deleted.  If -1, it never expires.
    # If $forward is given, the placeholder will point to it.

    local $debug = 0;
    $url = &untaintPath($url);
    my $urcFile = "$hndocs$url,urc";

    &HNMsg ("In deleteTree  url: $url\n") if $debug;

    #Get file information
    my %response = &parseURCFile("$urcFile");
    # Number of messages we are deleting.
    my $numMessages = $response{'NumMessages'};
      
    &printAssoc('Response', %response) if $debug;

    my $responses = $response {'Responses'};
    my $responsesPath = &untaintPath("$hndocs$responses");
    

    my $parent = $responses;
    $parent =~ s,/[^\/]*$,,;

    my $URL = $response {'URL'};
    my $base = $response {'BaseURL'};
    &HNMsg("URL: $URL\nbase: $base") if $debug;


    if (!$URL) {
      # Nothing to delete
      return 1;
    }

    if ($URL eq $base) {
	&HNUserError("Currently, $hnSystemName can NOT delete Base Articles!");
    }

    &logTransaction("Delete Tree", $url);

    #Remove tree of responses
    if (-d $responsesPath) {
	&lock_tree($responsesPath);
	&rmdirR($responsesPath);
	# Just unlock should be sufficient.
	&unlock_tree($responsesPath);
    }

    if ($expires || $forward) {
	my $date = &wwwdates::wtime(time, 'GMT');

	if ($expires) {
	    # Need to compute absolute date by adding $expires to current date.
	    #$response{'Expires'} = $expires;
	}

	if ($forward) {
	    $response{'Body'} = '';
	    $response{'Moved'} = $date;
	    $response{'LastMod'} = $date;
	    # Get remoteUser from global context.
	    $response{'MovedBy'} = $remoteUser if ($remoteUser);
	    $response{'MovedTo'} = "$forward";
	    $response{'NumMessages'} = 0;  # Nothing below this.
	}

	# should we have locked?  Maybe we don't care for deletion.
	&writeURC($urcFile, %response);

	# &clearResponseList($parent);
	&updateLists($base, $parent, -$numMessages);
    }
    else {
	# Delete the response itself.
	&deleteResponse($url, ($numMessages+1));
    }
}


sub deleteResponse {
    my ($url, $numMessages) = @_;
    # Delete a response at $url
    local($debug) = 0;

    $url = &untaintPath($url);
    my $urcFile = "$hndocs$url,urc";

    my %response = &parseURCFile($urcFile);
    &printAssoc("Deleting Response $url", %response) if $debug;

    my $deleted = $response {'Deleted'};
    my $URL = &untaintPath($response {URL});
    my $base = $response {'BaseURL'};
    &HNMsg("URL: $URL  \nbase: $base") if $debug;

    if (!$URL) {
      # Nothing to delete
      return 1;
    }
    if ($URL eq $base) {
	&HNUserError("Currently, $hnSystemName can NOT delete Base Articles!");
    }

    local($parentPath) = $urcFile;
    $parentPath =~ s,/[^\/]*$,,;  # Parent directory
    &HNMsg ("parentPath is $parentPath\n") if $debug;
    &lock($parentPath);

    my $responses = $response {'Responses'};
    my $bodyURL = $response {'Body'};
    &HNMsg ("bodyURL is $bodyURL\n") if $debug;

    # Adjusting the base URL for the document in case the URL and the base
    # are different
    #my $localhnget = $getURL;
    #if ($base !~ m|^$localhnget|) {
    #	$base =~ m|^(.*/$get)/|;
    #   $localhnget = $1;
    #}
    #&HNMsg ("localhnget = $localhnget\n") if $debug;

    my $parent = $responses;
    $parent =~ s,/[^\/]*$,,;
    &HNMsg ("parent = $parent") if $debug;

    local ($responsesPath) = "$hndocs$responses";
    &HNMsg ("responsesPath = $responsesPath") if $debug;

    local ($deleteOrReplace);
    # Check if responses exist.
    if (-d $responsesPath) {

        opendir(DIR, $responsesPath) ||
	    &HNError("Directory Error opening $responsesPath: $!");

	local(@dirFiles) = readdir DIR;
	closedir(DIR);
	# Look at just the response urc files.
	@dirFiles = grep (/^[0-9]+\.html,urc/, @dirFiles);
	&HNMsg ("dirFile is @dirFiles") if $debug;
	if (@dirFiles == 0) {
	    $deleteOrReplace = "Delete";
	} else {
	    $deleteOrReplace = "Replace";
	}
    } else {
        $deleteOrReplace = "Delete";
    }
    &HNMsg ("deleteOrReplace is $deleteOrReplace\n") if $debug;
    #==============================

    local($bodyPath);
    # Check if the body is local or a URL
    &HNMsg ("bodyURL = $bodyURL\n") if $debug;
    if ($bodyURL =~ m,$responses-(body|content).html$,) {
	$bodyPath = "$responsesPath-$1.html";
    }
    &HNMsg ("bodyPath = $bodyPath\n") if $debug;

    if ($deleteOrReplace eq "Delete") {
      # Proceed to delete
      &logTransaction("Delete", $url);

      # But first fix next and previous URC  files.
      &HNMsg ("urcFile = $urcFile\n") if $debug;
      &fixURCFiles($urcFile);
      
      unlink ($urcFile);
      if ($bodyPath) {
	unlink ($bodyPath);
      }
      if (-d $responsesPath) {
	&rmdirR($responsesPath)
      }
    } else {
      # Proceed to replace
      if ($bodyPath) {
	unlink ($bodyPath);
      } 
      
      if (!$deleted) {
	&lock($urcFile);
	my (%urc) = &parseURCFile($urcFile);
	$urc{'Body'} = "";
	local($date) = &wwwdates::wtime(time, 'GMT');
	$urc{'Deleted'} = $date;
	$urc{'LastMod'} = $date;
	
	&_writeURC($urcFile, %urc);
	&unlock($urcFile);
      }
    }
    &unlock($parentPath);

    &HNMsg ("parent = $parent") if $debug;
 
    # &clearResponseList($parent);
    &updateLists($base, $parent, -$numMessages);
}

sub fixURCFiles {
    my ($urcFile) = @_;
    local($debug) = 0;
    # This is way too much code for what little it does.

    my ($parentDir) = $urcFile;
    $parentDir =~ s|/[0-9]+\.html,urc$||;
    $parentDir =~ s|^$hndocs||;
    &HNMsg ("parentDir is $parentDir\n") if $debug;

    $filetofind = $urcFile;
    $filetofind =~ s|^$hndocs$parentDir/||;
    &HNMsg ("filetofind is $filetofind\n") if $debug;

    # calculating $counter and files under the directory
    @files = &responsesOf($parentDir);
    $counter = &computingURL($filetofind, @files);
    &HNMsg ("counter is $counter\n") if $debug;

    # get next and previous URC file from @files and $counter
    $nextindex = $counter + 1;
    $nextURC = &untaintPath($files[$nextindex]);
    if ($nextindex > @files){
	$nextURC = "";
    }
    &HNMsg ("nextURC is $nextURC\n") if $debug;

    $previousindex = $counter - 1;
    $previousURC = &untaintPath($files[$previousindex]);
    if ($previousindex < 0) {
	$previousURC = "";
    }
    &HNMsg ("previousURC is $previousURC\n") if $debug;

    my ($nextNum) = $nextURC;
    $nextNum =~ s|\.html,urc$||;
  
    my ($previousNum) = $previousURC;
    $previousNum =~ s|\.html,urc$||;
  
    &HNMsg ("previousNum is $previousNum\n") if $debug ;    
    # check whether next or previous response exists
    # if it exists, then fix URC file
    if ($nextURC){
	$nextURC = "$hndocs$parentDir/$nextURC";
	&HNMsg ("nextURC is $nextURC\n") if $debug;
	&lock($nextURC);
	my %next = &parseURCFile($nextURC);
	$next{'PreviousNum'} = $previousNum;
	&_writeURC($nextURC, %next);
	&unlock($nextURC);
    }
    if ($previousURC){
	$previousURC = "$hndocs$parentDir/$previousURC";
	&HNMsg ("previousNum  is $previousURC\n") if $debug;
	&lock($previousURC);
	my %previous = &parseURCFile($previousURC);
	$previous{'NextNum'} = $nextNum;
	&_writeURC($previousURC, %previous);
	&unlock($previousURC);
    }
}


sub computingURL {
    local($filetofind, @files) = @_;
    local($debug) = 0;

    &HNMsg ("delete-lib.pl filetofind is $filetofind\n") if $debug;
    local($current) = -1;
    &HNMsg ("index is @files\n") if $debug;
    foreach $file (@files){
	$current++;
	&HNMsg ("current is $current\n") if $debug;
	if ($filetofind eq $file){
	    &HNMsg ("internal current is $current\n") if $debug;
	    last;
        }
    }
    return $current;
}


sub rmdirR {
    my ($Dir) = @_;
    local $debug = 0;
    # Recursively remove a directory and contents == rm -r $Dir
    &HNMsg ("Deleting $Dir\n") if $debug; 
    # Open the directory to read the files
    opendir (DIR, "$Dir");
    my @files = readdir DIR;
    closedir DIR;

    my $file;
    foreach $file (@files) {
	&HNMsg ("File is \"$file\"\n") if $debug;
	if (($file ne ".") && ($file ne "..")) {
	    $file = &untaintPath($file);
	    if (-d "$Dir/$file") {
		&HNMsg ("Is Directory...\n") if $debug;
		&rmdirR("$Dir/$file"); 
	    } else {
		unlink("$Dir/$file");
	    }
	}
    }
    rmdir("$Dir");
}

1;
