# member-lib.pl - utilities for membership and password management
# Copyright Board of Trustees of the University of Illinois at UC, April 1995

# This file contains subs for member accessors, authentication, 
# access control, and password checking.
# Also see change-member-lib for utilities that support changes to this info.

require 'urc.pl';

##########################################
# Some options that should be in hnrc.

# Whether cookies are just userids.
# If making cookies, do the same.
$unsafeCookies = 0;


# This code is specific to using oraperl for maintaining member info.
# $externalSecurity must == 1 for this to work.  Either cookies or
# server access control above HyperNews may be used.
$UseOracle = 0;
if ($UseOracle) {
    eval '$Oraperl::safe = 1'       if $] >= 5;
    eval 'use Oraperl; 1' || die $@ if $] >= 5;

    #Set up Oracle variables to login
    $ENV{ORACLE_HOME} = ''; # path to oracle product
    $ENV{ORACLE_SID} = ''; # server id
    $Oracle_UserID_Password = '';  # userid/password ?
}
##########################################


require "urc.pl";


# Need to distinguish internal ids and displayable ids.

sub useridOK {
    my ($arg) = @_;
    # Check if $arg is OK as a userid. 

    # Convert multiple spaces or '_'s to a single '_'
    $arg =~ s{[_\s]+}{_}g if $allow_spaces_in_userids;
  
    if ($arg =~ m|^[A-Za-z0-9][A-Za-z0-9_\.\-]+$|) {
	return $arg;
    }
    return ($allowEmailUserID && &addressOK($arg));
}

sub untaintID {
    # Allow only characters of UserIDs or Email addresses to pass through.
    my ($id) = @_;
    $id =~ s{\s*}{}g if !$allow_spaces_in_userids;  # Remove whitespace.
    $id =~ m{^([\@A-Za-z0-9_=+%/\.\-]*)$};
    $id = $1;  # Untaint here
    return $id;
}
 
sub addressOK {
  my ($addr) = @_;
  # Return the address if it is a proper internet address, 
  # otherwise return a null string.
  # Must start with alphanumeric at least.
  # Must contain a '%'.
  if ($addr =~ m|^[A-Za-z0-9][A-Za-z0-9_=+%/\.\-]+[@][A-Za-z0-9_\.\-]+$|) {
      return $addr;
  }
  return '';
}


sub useridOf {
    my ($id) = @_;
    # Return current canonical (normalized) UserID for $id.
    # It may be an email address if allowed as userids.
    # $id need not be for a known member.  
    # If it is OK syntactically, it is returned.
    # It will be lowercased if $lower_case_IDs == 1.
    return '' if !$id;

    my %member = &getMember($id);
    $id = ($member{UserID} || (&useridOK($id) && $id));
    $id = lc $id if $lower_case_IDs;  # Should we do this here?
    return $id;
}


sub addressOf {
    my ($id) = @_;
    # Return the current email address of $id if a member.
    # Otherwise, return the $id if it is an email address.
    return '' if !$id;
    my %member = &getMember($id);
    return ($member{'Email'} || (&addressOK($id) && $id));
}


sub identifierOf {
    my ($userid) = @_;
    # Return a visible displayable identifier for the userid.
    # It might be an explicit Identifier, an email address, or userid.
    return '' if !$userid;
    my %member = &getMember($userid);
    return $member{Identifier} if $member{Identifier};

    my $email = &addressOf($userid);
    my $hideEmailAddress = $hideEmailAddresses;
    $hideEmailAddress ||= $member{Hide} eq 'Email';
    return $email if (!$hideEmailAddress && $email);

#    return $member{Name} if $member{Name};

    # Convert '_'s back to spaces, if allowed.
    $userid =~ s{_}{ }g if $allow_spaces_in_userids && !&addressOK($userid);

    # This always returns?  Because either we show email or userid.
    return $userid if !$hideUserIDs;  # but it might be an email address.

    # Otherwise?
    return $userid;
}


sub isAdmin {
  my ($id, $forum) = @_;
  # Check if $id is an administrator, either global or in forum.
  # It checks whether the $id is in the current @hnAdmin
  # in the current $hnGroup.  The owner of a $forum is also an admin.
  # This sub does not authenticate.
  &HNMsg("Checking if '$id' isAdmin") if $debug;
  $id = &useridOf($id);
  return 0 if !$id;
  my $admin;
  foreach $admin (@hnAdmin) {
      $admin = lc $admin if $lower_case_IDs;
      &HNMsg("&nbsp;&nbsp; Checking if $id is $admin") if $debug;
      return 1 if $id eq $admin;
      my %adminInfo = &getMember($admin);
      return 1 if ($id eq &useridOf($admin));
  } 

  # Test if user is owner of the forum.
  $forum = &url2hnpath($forum);
  return 0 if !$forum;
  my %urc = &parseURCFile("$hndocs$forum,urc");
  return 0 if !%urc;
  # &printAssoc("Forum URC", %urc) if $debug;
  my $owner = &useridOf($urc{From});
  return ($id eq $owner);
}


sub sustring {
    # Return a string containing the remote user name and attributes.
    my($remoteUser, $forum) = @_;
    my $result = '';
    if($remoteUser) {
	my $identifier = &identifierOf($remoteUser);
	$result = "<STRONG>User: $identifier</STRONG>";
	if (&isAdmin($remoteUser, $forum)) {
	    $result =~ s{User}{Administrator};
	}
    } 
    return($result);
}


###################################################################
# In-memory cache of member data that has been previously loaded.
# This is not real caching because we never throw out anything.
# For applications that would fill up memory without reclamation,
# cache only the last entry by setting $CACHE_MEMBERS = 0.
# For example, iterating through all the members should do that.
# To do: cache up to some number of members before throwing away
# the least used.

$CACHE_MEMBERS = 1;
%members_cache = ();

sub get_member_from_cache {
    my ($userid) = @_;
    my %member = ();
    return %member if !$userid;

    my $memberref = $members_cache{$userid};
    if ($memberref) {
	%member = %$memberref;
	&HNMsg("Found '$userid' in member cache.") if $debug;
    }
    return %member;
}

sub put_member_in_cache {
    my ($userid, %member) = @_;
    # Update or Store the %member under the $userid.

    return if !$userid || !%member;
    &printAssoc("Caching member info for '$userid'", %member) if $debug;

    # Maybe throw away the current cache.
    %members_cache = () if !$CACHE_MEMBERS;

    my $memberref = $members_cache{$userid};
    if ($memberref) {
	&HNMsg("Update cache entry for '$userid'.") if $debug;
	# Make it refer to the current value, which is necessary 
	# if member info was not modified by side effect.
	%$memberref = %member;
    }
    else {
	&HNMsg("Store new cache entry for '$userid'.") if $debug;
	$memberref = \%member;
	$members_cache{$userid} = $memberref;
	# Also cache under alternate email, if allowed as userid.
	$members_cache{$member{Email}} = $memberref if $allowEmailUserID;
    }
}

###################################################################

sub isMember {
    my ($id) = @_;
    # Return the $id if it is for a member in $hnGroup; 
    # return '' otherwise.
    # $id may be a user ID or email address, if allowed.
    
    &HNMsg("Checking if '$id' isMember") if $debug;

    return '' if !$id;
    $id = &untaintID($id);

    if ($externalSecurity) {
	# For now, just get the member info from outside.
	return &getMember($id);
    }
    else {
	return $id if -f "$hnPeople$hnGroup/$id";

	if ($lower_case_IDs) {
	    $id = lc $id;
	    return $id if -f "$hnPeople$hnGroup/$id";
	}
    }

    return '';
}


sub getMember {
    my ($userid) = @_;
    # Return a hash table for the member data of $userid

    # Note: The $userid might not be a member of the current
    # group ($hnGroup), but this routine will still return the data.

    local $debug = 0;

    return () if !$userid;       # Return empty list if no userid.

    my %member = ();

    &HNMsg("getMember: userid: '$userid'") if $debug;
#    &HNMsg("allowEmailUserID: $allowEmailUserID") if $debug;

    return () if !&useridOK($userid); #  Return if bad userid.

    # Untaint userid.
    $userid = &untaintID($userid);
        
    # UserIDs should be case insensitive for lookup, 
    # but be stored and displayed the original form.
    my $original_userid = $userid;
    $userid = lc $userid if $lower_case_IDs;

    # List of alternative UserIDs.
    my @alt_userids = ();

    %member = &get_member_from_cache($userid);
    return %member if %member;

    ####################################################

    if ($externalSecurity) {
	# Someone outside HyperNews may have the data.

        # We need a modular architecture for getting 
	# and putting this external data.

	if ($UseOracle) {
	    # This code is specific to using oraperl.
	    &HNMsg("Remote user: $remoteUser") if $debug;
	    &HNError("Remote user is unknown.") if !$userid;

	    my $lda = &ora_login($ENV{ORACLE_SID},
				 $Oracle_UserID_Password, '') || 
		warn "$ora_errno: $ora_errstr<BR>";  # HNError ?

	    my $crs = &ora_open($lda, "select nick_name, email
                         from participant 
			  where partic_id = \'$userid\'");
	    #  error is OK || warn "$ora_errno: $ora_errstr<BR>";

	    if ($crs) {
		my ($nick_name, $email) = &ora_fetch($crs);

		# Create a member record for use inside of HN.
		%member = ('UserID' => $userid,
			   'Identifier' => $nick_name,
			   'Email' => $email);
		# $authenticated = $remoteUser = $userid;  no no!
	    }
	}
	$userid = &untaintID($member{UserID});
	$original_userid = $userid;
	$userid = lc $userid if $lower_case_IDs;
	$userid =~ s{[_\s]+}{_}g if $allow_spaces_in_userids;
    }

    ####################################################

    else {
	# Not $externalSecurity so
	# HyperNews is supposed to have the data itself.

	# $userid might be an alias even if not an email address.
	# An alias might be a symbolic link, so it would be invisible here.
	# Or it might be an explicit UserID field different from file name.
	# Loop until the $userid eq $previous_userid. i.e. end of aliases.
	
	my $count = 0;
	my $previous_userid = '';

	{ # Top of loop; always do at least once.
	    return () if ++$count > 10;  # avoid infinite loop.

	    #&HNMsg("Parsing URCFile") if $debug;
	    if (-f "$hnPeople/$original_userid") {
		# Backup plan - try the original userid first.
		%member = &parseURCFile("$hnPeople/$original_userid");
	    }
	    elsif (-f "$hnPeople$hnGroup/$userid") {
		%member = &parseURCFile("$hnPeople/$userid");
	    }

	    # Check if a new user id exists as an alias.
	    $previous_userid = $userid;
	    $userid = &untaintID($member{UserID});
	    $original_userid = $userid;
	    $userid = lc $userid if $lower_case_IDs;
	    $userid =~ s{[_\s]+}{_}g if $allow_spaces_in_userids;

	    #####################################
	    # Exit loop here if no alias.
	    last if ($userid eq $previous_userid);
	    #####################################

	    push @alt_userids, $previous_userid;

	    # Try the cache again with this userid.
	    %member = &get_member_from_cache($userid);
	    last if %member;

	    redo;
	}
    }

    ####################################################

    # Commenting out the conditional breaks incoming mail restrictions.
    # and probably other things that depend on %member result being empty.

    if (%member) { 
	# &printAssoc ("member: '$userid'", %member) if $debug;

	# Should we write out member file if it is changed?
	# Certainly not for $externalSecurity.
	$member{Email} ||= (&addressOK($userid) ? $userid : '');
	$member{AltUserIDs} = join(', ', @alt_userids) if @alt_userids;

	&put_member_in_cache($userid, %member);
    }

    return %member;
}


sub putMember {
    my (%member) = @_;
    # Write out the new %member data.
    # Also update the cache.

    # Note: The $userid might not be a member of the current
    # group ($hnGroup), but this routine will still store the data.

    local $debug = 0;

    my $userid = $member{UserID};
    $userid = &untaintID($userid);
    # &ErrorIfTainted($userid);

    # Convert to lower case.
    $userid = lc $userid if $lower_case_IDs;

    # Convert multiple spaces or '_'s to a single '_'
    $userid =~ s{[_\s]+}{_}g if $allow_spaces_in_userids;

    &HNMsg("UserID: '$userid'") if $debug;

    &put_member_in_cache($userid, %member);

    if ($externalSecurity) {
      # Someone outside HyperNews may have the data.
      # Need a modular architecture for getting and putting this external data.
    }

    else {
	# We don't use $hnGroup here because the actual member info
	# is stored in the common $hnPeople dir.
	# We should have locked before starting any mods!!
	&writeURC("$hnPeople/$userid", %member);
    }
}
    
############################################################


sub moderatorTest {
    my ($remoteUser, $hnReadSF, $baseURL) = @_;

    local($debug) = 0;

    &HNMsg("baseURL :$baseURL\n") if $debug; 

    &HNMsg("authenticated: $authenticated\n") if $debug;

    my $remote_user_type = 
	&what_user_type($remoteUser, $authenticated, $baseURL); 

    &HNMsg("remote user type is:$remote_user_type\n") if $debug;
    &HNMsg("hnReadSF is:$hnReadSF \n") if $debug;
    if ($hnReadSF = 1) {
	if ($remote_user_type >= 2){
	    return 1;
	} else {
	    return 0;
	}
    } 
    elsif ($hnReadSF = 2) {
	if ($remote_user_type = 3){
	    return 1;
	} else {
	    return 0;
	}
    }

}

sub visibleTest {
}

#########################################################################
# Access Control

# The numeric identification and comparison of user classes in the following
# is confusing and cumbersome.  This needs to be redesigned from scratch.

# Perhaps we need to identify authenticated non-members as another type.
# Both remote user type and affected user type.
# For affected user, we also want to know if the authenticated remote user 
# is the same as the affected user.

sub what_user_type {
  my ($user,              # The user id of remote user.
      $authenticated,     # Whether the $remoteUser is authenticated.
      $baseURL            # The base URL for the document, if any.
      ) = @_;
  # Determinine the user type.  Types are:
  #  1-Non-members
  #  2-Members
  #  3-Administrators


  # Determine if $user is a non-member.
  if (!&isMember($user)) {
      # The user is a non-member?
      return (1);
  }

  # A member must be authenticated.
  if ($authenticated) {
      # Is the user an administrator but not mail admin?
      my $isAdmin = &isAdmin($user, $baseURL); # was && ($user ne $mail_admin);
      #&HNMsg("isAdmin = $isAdmin") if $debug;

      # User is a member.
      return (2 + $isAdmin);
  }
  return 1;
}


sub what_affected_type {
  my ($user,       	# The remote user id
      $affectedUser,    # The affected user id
      $baseURL          # The URL of the forum, if any.
     ) = @_;
   # Determine the affected user type.  Affected user types are:
   #  1-Non-members
   #  2-Same member as remote $user
   #  3-Different member from remote $user
   #  4-Administrator

   if (!&isMember($affectedUser)) {
       # The affected user is a non-member.
       return (1);
   }

   if (&isAdmin($affectedUser, $baseURL)) {  # including $mail_admin
       # The affected user is an administrator.
       return(4);
   } elsif ($affectedUser eq $user) {
       # Affected user and remote user are the same
       return(2);
   }
   # Affected user and remote user are different.
   return(3);
}

sub access1 {
    # Provide access control in HyperNews.
    # Causes exit if it fails, unless nonFatal is non-zero.
    # Returns $remote_user_type for success or 0 for failure.
    # Authentication is not done here - it must have already been done
    # by calling checkAuthenticated.  But we determine whether
    # authentication is required here.

    my ($whoCanAccess,		# See below.
	$whoCanBeAffected,	# Maximun user type that can be affected
	$affectedUser,		# The one whose info will be affected.
	$remoteUser,		# The one who is trying to do the command.
	$error_messageMore,	# What to say if an error occurs.
	$authenticated,		# Whether the $remoteUser was authenticated.
	$baseURL,		# Forum Base URL
	$Special_Case,		# if whoCanAccess==3, User can modify own info.
	$nonFatal 		# If 1, then just return 0 for failure.
	) = @_;

    #The $whoCanAccess parameter determines who has access to the command
    # Administrators always have access.
    # 1-Non-members + members
    # 2-Members only
    # 3-Administrators only

    #The $whoCanBeAffected parameter determines who can be affected by members
    # 1-Only non-members
    # 2-non-member + the member himself/herself
    # 3-any members + non-members, but not administrators
    # 4-All members including administrators
    local $debug = $debug || 0;

    $affectedUser = &useridOf($affectedUser);

    &HNMsg("
affected user: $affectedUser
remote user:   $remoteUser
authenticated: $authenticated") if $debug;

    &HNMsg("
whoCanAccess:     $whoCanAccess
whoCanBeAffected: $whoCanBeAffected
User can modify own info: '$Special_Case'") if $debug;

    # If no admin, no setup has been done yet, so assume admin is running it.
    return 3 if (!@hnAdmin);

    ######################
    # Do global access control by checking if user is allowed 
    # to access script in the path given by $ENV{SCRIPT_NAME}.
    # See hnrc for fixes of SCRIPT_NAME.
    # This is used now instead of server-based group access control.

    # This check is not redundant with other checks here because
    # the same script can be accessed via the Admin dir, to enable admin
    # functions, whereas it might be accessed via the HyperNews
    # dir when used by non-members.

	
    my $thisURL = &URLencode($ENV{REQUEST_URI});
    my $loginMsg = qq{
You can try to login first:
<A HREF="$loginURL?userid=$remoteUser&url=$thisURL">login here</A>.
} if ($ENV{REQUEST_METHOD} ne 'POST');
    # If POSTed, there should have been a userid and password in the form.

    # The Perhaps... is not right for non-server-authenticated.
    my $unknownUserError = qq{
 $hnSystemName does not know who the user is 
 and an unknown user is not permitted to perform this action.

 (Perhaps the server is not configured to authenticate properly, or it
 is not telling $hnSystemName who the REMOTE_USER is, 
 or perhaps the access restrictions are incorrect. Check the installation.) 

 $loginMsg
};
    ################################
    # Special access restriction based on the URL.

    if ($ENV{SCRIPT_NAME} =~ '/SECURED/' || $hnReadSF > 1) {
	# Authentication required.
	&HNUserError('You must be a member at this site. ' . 
		     $unknownUserError) if (!$authenticated);
    }

    if ($ENV{SCRIPT_NAME} =~ '(/.scripts/)|(/Admin/)' ) {
	# An admin is required.
	&HNUserError('You must be a member at this site. ' . 
		     $unknownUserError) if (!$authenticated);
	&HNUserError('You must be an administrator to use this program.')
	    if (!&isAdmin($remoteUser, $baseURL));
    }

    ################################
    # Determine what type of remote user this is.
    my $remote_user_type =  
	&what_user_type($remoteUser, $authenticated, $baseURL);

    $affected_user_type = 
	&what_affected_type($remoteUser, $affectedUser, $baseURL);

    &HNMsg("remote_user_type = $remote_user_type, 
affected_user_type = $affected_user_type") if $debug;

    #---------------------------------------------------------------
    # This is a hairy mess.  Need set relationships instead.

    if ($remote_user_type >= $whoCanAccess) {
	&HNMsg("Remote user is allowed to perform function, so far...") 
	    if $debug;

      if ($remote_user_type == 1) {
	# Remote User is non-member
	if ($affected_user_type == 1) {
	    &HNMsg("Non-member can affect non-member - presummably self.") 
		if $debug;
	  # If this is just so non-members can join, this is probably
	  # not the best way to do it.
	  goto OK;
	}
      } elsif ($remote_user_type == 3) {
	  &HNMsg("Administrator automatically has access.") if $debug;
	  goto OK;
      } else {
	  &HNMsg("Remote User is a non-admin member.") if $debug;
	
	if ($whoCanBeAffected >= $affected_user_type) {
	  # Here are some of the conditions that succeed.
	  # (1) only non-members >= non-members (1)
	  # (2) non-member or same member >= same member (2)
	  # (3) any member (non-admin) or non-member >= distinct members (3)
	  # (4) any member >= administrator (4)
	  &HNMsg("True: whoCanBeAffected >= affected_user_type") if $debug;
	  if (!$affectedUser || ($affectedUser eq $remoteUser) ||
	      $affected_user_type == 1) {
	    &HNMsg("No one else (or no member) is affected.") if $debug;
	    goto OK; 	  # A member can affect this user
	  }
	}
	# The only whoCanBeAffected values currently used are 2 and 3.
	# The following is a restriction of the above - maybe use instead.
	# The restriction is that $whoCanBeAffected value of 2 means
	# same member only.
	# But it messes up. e.g. Member-only forums disallow writing messages.

	if ($whoCanBeAffected == 2 && $affected_user_type == 2) {
	    &HNMsg("Member can only affect self.") if $debug;
	    goto OK;
	}
	elsif ($whoCanBeAffected == 3 && $affected_user_type <= 3) {
	    goto OK;
	}
      }
    }

    if ($Special_Case && ($whoCanAccess == 3) && ($affected_user_type == 2)) {
	&HNMsg("Special case: User can modify his own information in level 3")
	    if $debug;
	return $remote_user_type;
    }

    ###############################
    # Access check failed.

    return 0 if $nonFatal;
    # The following status may cause browser to display authentication dialog.
    # We don't really want that.
    $HNStatus = '403';

#    if ($affectedUser) {
#	$userAffected = "<STRONG>User Affected: </STRONG> $affectedUser <br>";
#    }

    if ($remote_user_type == 1) {
	&HNMsg("Unknown user, but member was required.") if $debug;

	if ($remoteUser eq 'UnknownUser') {
	    # This is probably from add-article.pl
	    &HNUserError ($unknownUserError);
	}

	&HNMsg("if \$remoteUser is a member, must be a password problem.") 
	  if $debug;
	if (&isMember($remoteUser)) {

	    my $fixPasswordMsg;

	    if (!$manualSecurity) {
		$fixPasswordMsg = qq{
If you believe your $UserIDorHandle and Password at $hnSystemName are correct,
please try changing it on the <A
HREF="$SECUREDeditMemberURL?$remoteUser">secured member form</A> (you
will be prompted by a dialog box for your $UserIDorHandle and Password), and
then try again.  };
	    }
	    else {
		my $adminEmail = &addressOf(@hnAdmin[0]);
		$fixPasswordMsg = qq{
If you believe your $UserIDorHandle and Password at $hnSystemName are correct,
please ask the $hnSystemName administrator, $adminEmail, to reset your
password.  
};
	    }
	    
	    # It should detect when the user is trying to register.
	    &HNUserError( qq{
You must enter your password to prove you are '<strong>$remoteUser</strong>'.
If you entered your password, perhaps you typed it wrong.
If you are trying to register as '$remoteUser', perhaps that
$UserIDorHandle is already registered by someone else.
$securedMemberForm
$helpMsg
});
	}
	else {
	  &HNMsg("remoteUser is a non-member") if $debug;
	  &HNUserError( qq{
$unknownUserError
$helpMsg
});
	    
	}

    }

    &HNMsg("$remoteUser is a member, so must be only admin may access.") 
	if $debug;
    &HNUserError ("Permission denied. <P> \n$error_messageMore\n");

  OK:
    return $remote_user_type;
}



###################################
# Cookies

$HNSalt = 'HN';  # Two characters - become visible salt - could be random.
$csCookiePassword = &checksum($HNCookiePassword || 'whatever');

sub checksum {
  my ($string) = @_;
  return unpack("%32C*", $string);  # 32 bit check sum
}


sub makeUserCookie {
  my ($user, $cpassword) = @_;
  # Make a string that concatenates the secret HNCookiePasswd with
  # the user's encrypted password.  This is a crude, half-and-half
  # security measure that is half server-secret and half user-secret.

  my %member = &getMember($user);
  # If there is no password, use the userid.
  $cpassword ||= $member{'Password'} || &useridOf($user);

  return if !$csCookiePassword;
  # &HNMsg("csCookiePassword: '$csCookiePassword'") if $debug;
  my $cpasswds = "$csCookiePassword$cpassword";
  my $doublecryptedpass = crypt($cpasswds, $HNSalt);
  return $doublecryptedpass;
}


sub assignUserCookie {
  my ($user, $cookie_received, $send_cookie) = @_;
  # Create a cookie to remember the $user and encrypted password.
  # Or create a cookie to forget the last $cookie_received.
  # If $send_cookie is true, we can send a cookie if user wants one.
  # See hn-lib for where the cookie in $HNUserPassCookie is written.

  local $debug = $debug || 0;

  return if $UseCookies eq 'never';  # Just ignore.
  &HNMsg(qq{In assignUserCookie:
User: $user
Received: '$cookie_received'
Send: $send_cookie
}) if $debug;

  my $dcpass = &makeUserCookie($user, $cpassword);

  if ($UseCookies eq 'sometimes') {
      &HNMsg("Find out if user really does want cookies.") if $debug;
      my %member = &getMember($user);
      if (!$member{SessionLength} || $member{SessionLength} eq 'none') {
	# no cookies here.
	&HNMsg("...oops. This user does not want cookies.") if $debug;
	$dcpass = '';
      }
  }
  # else $UseCookies eq 'always'

  &HNMsg("New cookie: '$user:$dcpass'") if $debug;
  if ($cookie_received && 
      ($cookie_received ne "$user:$dcpass")) {
    &HNMsg("Clear previous cookie, at least.") if $debug;
    $HNUserPassCookie = qq{HN_USER_PASS=};
  }

  if ($dcpass && $send_cookie) {
    if (!$cookie_received  ||
	($cookie_received ne "$user:$dcpass")) {
      # Cause the HN_USER_PASS cookie to be sent to client - see hn-lib.pl
      $HNUserPassCookie = qq{HN_USER_PASS=$user:$dcpass};
    }
  }
  &HNMsg("Assigned cookie: '$HNUserPassCookie'") if $debug;
}


################################
# Authentication check.

sub checkAuthenticated {
    # Check whether the user has been authenticated.
    # Every script should call this, in case the authentication is required.
    # But this routine doesn't require authentication - 
    #  access1 does that, if appropriate.
    # This only checks if the user has been authenticated by the server, or
    # it authenticates against form supplied password, or checks a cookie.

    # Uses global vars: $authenticated, $ENV{REMOTE_USER}
    # Sets global vars: $remoteUser, $remotePassword, $authenticated

    my ($user1, $password1, $user2, $password2) = @_;

    # * First try user2 from form. 
    # user2 might be an explicit admin doing something for (or to) the user.

    # * Then try user1 from form. 
    # user1 might be an explicit user login.

    # * Then try $ENV{REMOTE_USER} from server authentication.
    # Doing this here means an admin cannot post for another
    # member without maybe becoming that member.
    # But doing it form check after this means that one cannot login.

    # Big problem: non-admin authenticates via cookies, 
    # and that person is allowed to edit admin info !?!

    # * Then try Cookie.  
    # A Cookie may be sent by a browser due to a previous authentication
    # (that resulted in a Set-Cookie from HyperNews)
    # or due to some external security mechanism.

    # This order of checking allows an explicit form-based 'login' (user2)
    # to override an authentication sent automatically by the browser.  

    # Debugging before making the cookie causes the cookie to not be sent.
    local $debug = $debug || 0;

    &HNMsg("Check Authentication of 
    user1: '$user1' pw1: '$password1'
 or user2: '$user2' pw2: '$password2'") if $debug;

    # One problem: users with no password authenticate too easily.
    # This is OK except for making cookies.  So don't make a cookie
    # if there is no password (unless initial admin). 
    # But it's OK if server authenticated.

    if (!$authenticated && $user2) {
      if (&passwordMatch($user2, $password2)) {
	# Don't allow $mail_admin to have a blank password.
	if (($user2 eq $mail_admin) && !$password2) {
	  &HNUserError("
The password is blank for '$mail_admin', the  Mail Administrator of 
$hnSystemName.  A blank password is not allowed.  
See \$mail_admin_password in the HyperNews configuration (hnrc).");
	}
	$authenticated = 'by form(2)';
	$remoteUser = $user2;
	$remotePassword = $password2;
	&HNMsg("User2 '$user2' authenticated with pw: '$password2'.") 
	  if $debug;
	# Special check for mail administrator.
	if ($remoteUser eq $mail_admin && $user1) {
	  # Mail admin has been authenticated, 
	  # so proceed as if $user1 is the $remoteUser, if there is one.
	  $authenticated = 'by mail admin';
	  $remoteUser = $user1;
	}
      }
      else {
	&HNMsg("Failed matching '$user2' with pw '$password2'") if $debug;
      }
    }

    if (!$authenticated && $user1) {
      if (&passwordMatch($user1, $password1)) {
	# Allow blank password here. ($user1 eq $mail_admin && !$password1)
	$authenticated = 'by form(1)';
	$remoteUser = $user1;
	$remotePassword = $password1;
	&HNMsg("User1 $user1 authenticated with pw: '$password1'.") 
	  if $debug;
      }
      else {
	&HNMsg("Failed matching '$user1' with pw '$password1'") if $debug;
      }
    }


    if (!$authenticated && 
	# Has user been authenticated by server?
	$ENV{REMOTE_USER}) {
	# if $ENV{REMOTE_USER} is assigned, then believe it.
	$authenticated = 'by server';
	$remoteUser = $ENV{REMOTE_USER};  
	$remotePassword = '';
	&HNMsg("Remote user authenticated by server: $remoteUser") 
	    if $debug;
    }

    ######################
    # Use or make a cookie.  
    # Try to verify cookie if sent.
    # Send a new cookie if needed.

    # Whether a new cookie might be made.
    my $maybe_make_cookie = $UseCookies ne 'never';
    my $cookie_received;  # Remember whether a cookie was received.

    # Check whether a cookie was received from the client.
    # Do this even if already authenticated above, to find out if new cookie
    # is still needed.
    if (($UseCookies ne 'never') &&
	($ENV{'HTTP_COOKIE'} || $ENV{'COOKIE'})) {
	# &HNMsg("Got Cookie: '$ENV{'HTTP_COOKIE'}' or '$ENV{'COOKIE'}'") 
	#  if $debug;
	require "formlib.pl";

	my %cookieData = &parse($ENV{'HTTP_COOKIE'} || $ENV{'COOKIE'}, '; ');
	my $user_pass_cookie = $cookieData{'HN_USER_PASS'} || 
	    $cookieData{'userid'}; # case sensitive!!
	&HNMsg("user pass cookie stage 1: '$user_pass_cookie'") if $debug;

	# Get first one on the list.
	if (ref $user_pass_cookie) {
	    my @cell = @$user_pass_cookie;
	    $user_pass_cookie = $cell[0];
	}

	&HNMsg("user pass cookie stage 2: '$user_pass_cookie'") if $debug;
	$cookie_received = $user_pass_cookie;

	#while (ref $user_pass_cookie) {
	# Get the last one on the list.  That might be the wrong one.
	#  my @cell = @$user_pass_cookie;
	#  &HNMsg("cell: '$cell[0]'") if $debug;
	#  $user_pass_cookie = $cell[1];
	#  &HNMsg("next cell: '$user_pass_cookie'") if $debug;
	#}
	#&HNMsg("user pass cookie stage 3: '$user_pass_cookie'") if $debug;

	my ($cookie_userid, $cookie_dcpass) = split(/:/, $user_pass_cookie);
	my %cookie_member = &getMember($cookie_userid);

	if ($UseCookies eq 'never' || 
	    (($cookie_member{SessionLength} eq 'none')
	     # But OK if always UseCookies.
	     && ($UseCookies ne 'always'))) {
	    &HNMsg("Don't use the cookie, and don't make another one.") 
		if $debug;
#	    $maybe_make_cookie = 0;
	}
	elsif ($cookie_userid) {
	    if ($unsafeCookies) {
		$authenticated = 'by unsafe cookie';
		$remoteUser = $cookie_userid;
		&HNMsg("Authenticated with unsafe cookie: $authenticated") 
		    if $debug;
		$maybe_make_cookie = 0;  # Don't need to send another.
	    }
	    else {
		# Authenticate the cookie.
		my ($doublecryptedpass) = &makeUserCookie($cookie_userid);
		&HNMsg("Cookie UserID: '$cookie_userid'
Double Encrypted Password: '$doublecryptedpass'") if $debug;
		if ($cookie_dcpass eq $doublecryptedpass) {
		    &HNMsg("Believe cookie, 
but don't use it if newly authenticated userid should override.") 
			if $debug;
		    # Or perhaps we need an
		    # option to override server authentication.
		    if (!$authenticated) {
			$authenticated = 'by cookie';
			$remoteUser = $cookie_userid;
			&HNMsg("User '$remoteUser' Authenticated with cookie.") 
			    if $debug;
#			$maybe_make_cookie = 0;  # Don't need to send another.
		    }
		    else {
			&HNMsg("New authenticated user.
 Maybe make new cookie if '$remoteUser' != '$cookie_userid'.")
			    if $debug;
			# Might be same user with alias !!
#			$maybe_make_cookie = ($remoteUser ne $cookie_userid);
		    }
		}
		else {
		    &HNMsg("User '$cookie_userid' Not Authenticated with cookie.")
			if $debug;
		    # The user or password might have changed.
		    # maybe_make_cookie = 1, if ok otherwise.  see below.
		    # Use the userid anyway, if remoteUser not previously set.
		    if (!$remoteUser) {
			$remoteUser = $cookie_userid;
		    }
		}
	    }
	}
    }
    else {
	&HNMsg("No cookie from browser, so ...") if $debug;
    }

    # Done checking authentication.
    # Normalize the userid of $remoteUser.

    $remoteUser = &useridOf($remoteUser);
    &HNMsg("Authenticated: '$authenticated'  Remote User: '$remoteUser'")
	if $debug;

    ####################################################
    # If cookies are allowed and we have authenticated,
    # and we need a cookie, then send a cookie.

    my %member = &getMember($remoteUser);

    # Test if cookie should be sent.
    $maybe_make_cookie &&=
      ($authenticated && 
       ($remotePassword || $ENV{REMOTE_USER} || &isAdmin($remoteUser)) &&
       ($remoteUser ne $mail_admin)  # mail_admin never gets a cookie
      );

    &assignUserCookie($remoteUser, $cookie_received, $maybe_make_cookie);

    ##################
    # Finish up.
     
    return $authenticated;
}


sub check_userid_email {
  my ($userid, $email) = @_;
  # Cause error if $userid is not for same user as $email.
  $userid = &useridOf($userid);
#  &HNMsg("userid = '$userid'") if $debug;
  if (&isMember($email)) {
    my $email_userid = &useridOf($email);
    if ($email_userid ne $userid) {
      &HNUserError("Email address '$email' is for user
'$email_userid', which is different from user '$userid'.");
    }
  }
}


#################################
# Password matching


# Setting $externalSecurity = 1 could mean the user has been
# authenticated before entering HyperNews, e.g. by the server.  But if
# not yet authenticated, we check whether a form-supplied password matches
# via an external program of some kind.  Below are a couple functions
# that do this kind of check.  (These should probably be in external
# modules that are included only if appropriate for a configuration.)

# Whatever is assigned to $external_check_password is used.
# e.g. $external_check_password = \&check_password_via_socket;
# Also you must set $externalSecurity = 1

my $external_check_password;  # (Should be moved to hnrc.)


#################################
# Check userid and password via a socket call.

my $CHECK_PASSWORD_ADDRESS = '';  # IP number
my $CHECK_PASSWORD_PORT = 5000;

use Socket;

sub check_password_via_socket {
    my ($userid, $password) = @_;

    # require Socket; import Socket;  This doesnt work
    my $addr = $CHECK_PASSWORD_ADDRESS;
    my $port = $CHECK_PASSWORD_PORT;

    my ($iaddr, $paddr, $proto, $line);

    if ($port =~ m{\D}) {
	$port = getservbyname($port, 'tcp');
    }
    return "No port" unless $port;
    $iaddr = inet_aton($addr)    ||
	return "no host: $addr";
    $paddr = sockaddr_in($port, $iaddr);

    $proto = getprotobyname('tcp');
    socket(SOCK, PF_INET, SOCK_STREAM, $proto) or return "socket: $!";
    connect(SOCK, $paddr) ||
	return "connect: $!";
    select((select(SOCK) , $| = 1)[0]);	# autoflush

    print SOCK "passwordCheck $userid#IGC#$password#IGC#none#IGC#\n";
    my $result = "";
    if ($line = <SOCK>) {
        $result = $line;
    }
    close(SOCK)     or return "close: $!";
    return $result;
}

###################################
# Check the password via a call to an outside program.

my $CHECK_PASSWORD_PATH =
        '/usr/ns-home2/cgi-bin/HyperNews/.scripts/checkPassword';
 
sub check_password_via_program {
    my ($userid, $password) = @_;
    # This needs to be customized for each installation.
    # What dir are we in?
    $userid = &untaintID($userid);
    # untaint the password, carefully.
    my $tmp = $password;
    $tmp =~ m{([^\"]*)};
    $password = $1;
    my $result = `$CHECK_PASSWORD_PATH "$userid" "$password" "x"`;
    return $result;
}


#############################################################################33


sub passwordMatch {
    my ($userid, $password, $current_password_c) = @_;
    # Test whether the password of $userid is $password.
    # If $current_password is supplied, that is the password to compare with.
    # Return 0 if $userid is not a member and $current_password_c is null.

    return 0 if !$userid;  # No userid is not ok.

    local $debug = 0;

    &HNMsg("Checking password of '$userid'.   Entered: '$password'") if $debug;
 
    if ($externalSecurity && $external_check_password) {
	my $result = &$external_check_password($userid, $password);
        &HNMsg("External password check returned: $result") if $debug;
        return 1 if !$result;
        return 0;
    }

    if (!$current_password_c) {
        my %member = &getMember($userid);
	# Non-members never match.  (except blank pw??)
	return 0 if !%member;   # was: return ($password eq '')
        $current_password_c = $member{'Password'};  # This is encrypted
    }

    if ($current_password_c) {
      # Blank passwords are encrypted too.

      my $salt;
      # Using MD5 crypt (or extended DES?), password starts with $1$
      $salt1 = substr($current_password_c,
                 ($current_password_c =~ m,^\$1\$, ? 3 : 0), 2);
      # But using the current password directly seems to work fine too.
      $salt2 = $current_password_c;
      # If your system uses MD5 crypt and you previously created passwords
      # with only 2-char salt, as in salt1, then salt2 will not work.
      # FreeBSD Unix crypt in Perl seems to always use MD5 encryption
      # even when it is given a 2-char salt, contrary to the crypt man page.

      my ($cryptedpass1) = crypt($password, $salt1);
      my ($cryptedpass2) = crypt($password, $salt2);

      &HNMsg ("
Entered password is '$password'
Encrypted with salt1 it is  '$cryptedpass1'
Encrypted with salt2 it is  '$cryptedpass2'
Current crypted password is '$current_password_c'
") if $debug;

        # my $cryptedpass = $cryptedpass2;
        # If salt2 doesn't work, try the next line.
        # my $cryptedpass = $cryptedpass1;
      # Heck, try both.

      if ($current_password_c eq $cryptedpass2 ||
          $current_password_c eq $cryptedpass1) {
        # Password matches, so this guy really knows who he is.
        return 1;
      }
    }
    elsif (!$current_password_c && ($password eq '')) {
        # $current_password_c should only be blank if manually set to blank.
        # Assume this is OK and return positive.
        # &assignUserCookie($userid);  dont do this
        &HNMsg("Blank password is OK") if $debug;
        return 1;  # Empty password allowed, even if $requirePasswords.
    }

    # Failed to match.
    &HNMsg("Failed to match password.") if $debug;
    return 0;
}
  

# Not used.
sub check_passwd {
    my ($cpasswd,$passwd) = @_;
    my ($salt);

    $salt = substr($cpasswd,0,2);

    return(crypt($passwd,$salt));
}


1;

