#!/usr/local/bin/perl
# response-lib.pl - HyperNews response library.
# Copyright Board of Trustees of the University of Illinois at UC, April 1995

require "urc.pl";
require "icon-lib.pl";
require "lock.pl";


sub clearResponseList {
  my ($dir) = @_;
  # Clear out the response list for the directory $dir, relative to $hndocs.
  # Next time the file is accessed for display, then it will get regenerated.

  &HNMsg("clearResponseList dir: $dir") if $debug;
  if (!$dir) {
    &HNError("HyperNews is broken - please report this bug.
 \$dir is null in call of clearResponseList");
  }
  return if (!-d "$hndocs$dir");  # Nothing to clear.
  
  &lock("$hndocs$dir");
  open(RFH, ">$hndocs$dir/responses.html-new") ||
    &HNError("Problem creating $hndocs$dir/responses.html-new");
  print RFH "#numResponses: -1\n";
  close RFH;
  my $filename = "$hndocs$dir/responses.html";
  &moveItSafely($filename, "$filename-old", "$filename-new");
  &unlock("$hndocs$dir");
}


sub updateResponseList {
    local($dir) = @_;
    # Rebuild the response list for the named directory, relative to $hndocs.
    # It is used when a subresponse is created so that the
    # list of responses includes the list of subresponses.
    # It is also be used when responses are deleted, or made invisible

    # Too much is local - should be 'my'!
    local($file, @files);
    local($title, $userName, $userURL, $date);
    local($responseLine, $deleted, $moved, $visible);
    local($upURL, $upRel);
    local($ok) = 1;

    local(%icons) = &parseURCFile($hniconsTable);

    my ($messageFile) = "responses.html";

    if (!$responses) {
      &HNError("HyperNews is broken - please report this bug.
 \$dir is null in call of updateResponseList");
    }
    &lock("$hndocs$dir");

    # Now that we have the lock, we check whether the update is still
    # needed.  Someone else might have updated while we were waiting
    # for the lock to be made.

    open(RFH, "<$hndocs$dir/$messageFile") ||
      &HNError("Problem reading $hndocs$dir/$messageFile");
    my $nextline = <RFH>;
    if ($nextline =~ m"^#numResponses: ([0-9-]*)") {
      my $numResponses = $1;
      if ($numResponses != -1) {
	# We could check the date on the file..
	&unlock("$hndocs$dir");
	return 0;
      }
    }

    open(RFH, ">$hndocs$dir/$messageFile-new") ||
      &HNError("Problem creating $hndocs$dir/$messageFile-new");

    opendir (DIR, "$hndocs$dir") ||
      &HNError("Problem reading $hndocs$dir");
     # &HNMsg ("Opendir $hndocs$dir\n") if $debug;

    @files = readdir DIR;
    closedir DIR;

    # Look at the response urc files in numeric order.
    @files = grep (/^[0-9]+\.html,urc$/, @files);

    # Thanks to davidh@psiphi.org for pushing top-level reversal.
    if (($reverseMessages == -1) ||
	(($reverseMessages == 1) && ($dir !~ m{/[0-9]+$ }x))) {
	# -1 is full reverse; 1 is reverse only top-level messages.
	# The regexp looks for a non-message number at the end.
	@files = sort {$b <=> $a} @files;
    }
    else {
      @files = sort {$a <=> $b} @files;
    }
    # &HNMsg ("Files: @files\n") if $debug;

    # First line is special.
    local ($numResponses) = @files + 0;
    $ok &= print RFH "#numResponses: $numResponses\n";

    # Record type and fields of each record.
    #$ok &= print RFH "Message|Num|Title|Author|Deleted|Moved|Visible|UpRel|Date|NumReplies|NumAllReplies|LastReplyDate\n"; # new way

    foreach $file (@files) {
	# &HNMsg ("File: $file\n") if $debug;
	if ($file =~ m|(.*)\.html,urc$|) {
	    my $root = $1;

	    my %urc = &parseURCFile("$hndocs$dir/$root.html,urc");

	    my $URL = $urc{'URL'};
	    next if (!$URL);

	    $title = $urc{'Title'} || 'Untitled';
	    my $userid = $urc{'From'};

	    # Use member info if available.
	    my %member = &getMember($userid);
	    $userid = &useridOf($userid);
            my $email = &addressOf($userid);
            my $identifier = &identifierOf($userid);

	    my $hideEmailAddress = $hideEmailAddresses;
	    $hideEmailAddress ||= $member{Hide} eq 'Email';
 
	    if ($viewMembers) {
		$userURL =  
		    ((!$hideUserIDs && $userid) ? "$viewMemberURL?$userid" :
		     (!$hideEmailAddress && $email) ? "$viewMemberURL?$email" 
		     : '');
		     # was $member{'UserURL'} || $userURL
	    }

            $userName = $member{Name} || $urc{Name} || $identifier;
            if ($email && ($userName eq $email)) {
                    $userURL = "mailto:$email";
            }
 
	    $deleted = $urc{'Deleted'};
	    $moved = $urc{'Moved'};
	    $visible = $urc{'Visibility'};
	    $upRel = $urc{'UpRel'};
	    $date = $urc{'Date'};
	    $annotationType = $urc{'AnnotationType'};
	    #$referenceURL = $urc{'ReferenceURL'};

	    if (!$annotationType 
		|| $annotationType eq 'Message'
		|| $annotationType eq 'Response') {

		$responseLine = 
		    "<DT> $root.\n" .
		    &makeResponseLine($URL, $title, 
				      $userURL, $userName, 
				      $deleted, $moved, $visible, 
				      $upRel, $date) . "\n</DT>";

		#$responseLine = &encodeRecordLine
		#  ($root, $title, $userName, 
		#   $deleted, $moved, $visible, $upRel, $date);


		$ok &= print RFH "$responseLine\n";
		if (-d "$hndocs$dir/$root") {
		    # Include responses to this response.
		    $ok &= print RFH "#include \"$root/$messageFile\"\n";
		}
	    }
	}
    }

    if (!$ok) {
	&HNError("Problem writing to $hndocs$dir/$messageFile-new");
    }

    close RFH;

    my $filename = "$hndocs$dir/$messageFile";
    &moveItSafely($filename, "$filename-old", "$filename-new");

    &unlock("$hndocs$dir");

    if (@files == 0) {
	# No responses in this directory.

	# Check if parent is marked for deletion.
	%in = &parseURCFile("$hndocs$dir.html,urc");
	local($ISdeleted) = $urc{'Deleted'};
	if ($ISdeleted) {

	    # Delete this directory, if it is for a response.
	    if ("$dir" =~ m,/[0-9]+\$,) {
		&deleteResponse("$hndocs$dir.html,urc");
	    }
	}
    }
}


sub makeResponseLine {
    my ($URL, $title, $userURL, $userName,
	  $deleted, $moved, $visible, $upRel, $date) = @_;
    my $responseLine;

    my $location = $icons{$upRel} || $icons{'Note'}; 
    if ($upRel) { $upRel .= ':'; }

    if ($useHNIcons) {
	$responseLine .= "<IMG src=\"$hniconsURL/$location\" ALT=\"$upRel\" WIDTH=$relationIconXsize HEIGHT=$relationIconYsize>";
    }

    $responseLine .= "<A HREF=\"$URL\">$title</A>";
    my $userLine;
    if ($userURL) {
	$userLine = "(<A HREF=\"$userURL\">$userName</A>)";
    }
    else {
	$userLine = "($userName)" if $userName;
    }

    $responseLine .= " $userLine";

    if ($deleted) {
	$responseLine .= " (Deleted)";
    }
    if ($moved) {
	$responseLine .= " (Moved)";
    }
    
    if ($visible eq "Invisible") {
	$responseLine .= "<Invisible>";
    }

    $responseLine .= "<DATE=$date>";

    return $responseLine;
}


sub updateReferenceList {
    local($dir) = @_;
    # Rebuild the reference list for the named directory.
    local($file, @files);
    local($title, $lowerTitle, $URL, $date);
    local($referenceLine, $deleted);
    local($upURL, $upRel);
    local(@list); # of refs.

    opendir (DIR, "$hndocs$dir");
    # &HNMsg ("Opendir $hndocs$dir\n") if $debug;

    @files = readdir DIR;
    closedir DIR;

    # Look at the reference urc files in numeric order.
    @files = grep (/^ref[0-9]+\.html,urc$/, @files);
    # &HNMsg ("Files: @files\n") if $debug;

    foreach (@files) {
	$file = $_;
	# &HNMsg ("File: $file\n") if $debug;
	my %urc = &parseURCFile("$hndocs$dir/$file");

	$title = $urc{'Title'};
	$URL = $urc{'URL'};
	$referenceURL = $urc{'ReferenceURL'};
	$deleted = $urc{'Deleted'};
	$upRel = $urc{'UpRel'};
	$date = $urc{'Date'};

	if ($referenceURL) {
	    local($root) = $file;
	    $root =~ s/\.html,urc$//;
	    local($body) = '';
	    # There must be a better way.
	    open (BH, "<$hndocs$dir/$root-body.html");
	    while (<BH>) {
		$body = "$body\n$_";
	    }
	    close BH;

	    # localhnget is assigned by add-lib.pl
	    $referenceLine = &makeReferenceLine
		("$localhnget$relativePrefix.html", 
		 $title, $referenceURL, $body, '', $upRel, $date);
	    # $lowerTitle = lc($title); # causes problems for perl 4.
	    push (@list, "$lowerTitle<$referenceLine\n");
	}
    }

    @list = sort @list;
    local(@refs);
    # Following line causes problems for perl 4.
    # @refs = map { substr ($_, 1+index ($_, "<")) } @list;

    open(RFH, ">$hndocs$dir/newReferences.html");
    # print RFH " \@refs\n";  # Causes warning.
    close RFH;

    rename ("$hndocs$dir/newReferences.html",
	    "$hndocs$dir/references.html");
    chmod "$hnfilemode", "$hndocs$dir/references.html";

}


sub makeReferenceLine {
    local($URL, $title, $referenceURL, $body, $deleted, $upRel, $date) = @_;
    local($referenceLine) = "<LI> ";

    if ($upRel) {
	my %icons = &parseURCFile($hniconsTable);
	local($location) = $icons{"$upRel"}; 
	$referenceLine .= "<IMG src=\"$hniconsURL/$location\" ALT=\"$upRel:\">";
    }

    if ($referenceURL) {
	$referenceLine .= "<A HREF=\"$referenceURL\">$title</A>";
    }
    else {
	$referenceLine .= "$title";
    }

    if ($body) {
	$referenceLine .= "\n - $body";
    }

    if ($deleted) {
	$referenceLine .= " (Deleted)";
    }

    return $referenceLine;
}



sub responsesOf {
    local($dir) = @_;
    # return a sorted array of responses in the directory named by $dir
    local(@files);
    local($tmp) = "$hndocs$dir";
    opendir (DIR, "$tmp");
    @files = readdir DIR;
    closedir DIR;
    @files = grep (/^[0-9]+\.html,urc$/, @files);
    @files = sort {$a <=> $b} @files;
    return @files;
}


1;



