# subscribe-lib.pl - utilities for subscription management
# Copyright Board of Trustees of the University of Illinois at UC, April 1995

require "urc.pl";
require "member-lib.pl";

# Todo: Don't email a new message to anyone who is auto-subscribed by another
# message from the same origin as the new message since they probably
# already got the message from that same origin.  More general solution
# is I-HAVE/SEND-ME protocol even for mail delivery.  


sub getSubscription {
    my ($dir, $userid) = @_;
    # Get the subscription for a single $userid.  Return:
    #  ($aSubscriber, $subscribeFlag, $frequency, "Direct" or "Not Specified");

    local $debug = 0;

    if (!$userid) {
	return  ('', '', '', "Not Specified"); 
    }

    $userid = &useridOf($userid);
    &HNMsg("normalized user: '$userid'") if $debug;

    &HNMsg("Parsing: $hndocs$dir.html,urc") if $debug;

    my %urc = &parseURCFile("$hndocs$dir.html,urc");
    &printAssoc("URC", %urc) if $debug;

    my $responses = $urc{'Responses'};
    my $author = $urc {'From'};

    my ($aSubscriber, $subscribeFlag, $frequency) = ("","","");

    open (ML, "<$hndocs$responses/list");
    my $nextline = <ML>;
    while ($nextline && !$subscribeFlag) {
	($aSubscriber, $frequency) = split(':', $nextline);
	# This translation can get expensive.
	$aSubscriber = &useridOf($aSubscriber);
	&HNMsg("Subscriber: $aSubscriber") if $debug;
	
	if ($aSubscriber eq $userid) {
	    $frequency =~ s,[ \n],,g;
	    $frequency ||= 'Immediate';
	    $subscribeFlag = $frequency;
	    &HNMsg("found it") if $debug;
	}
	$nextline = <ML>;
    }
    close(ML);
    # If not explicitly subscribed, check author.
    if (!$subscribeFlag) {
	$author = &useridOf($author);
	&HNMsg("Check author: $author") if $debug;
	if ($author eq $userid) {
	    $aSubscriber = $author;
	    $frequency = 'Immediate'; 
	    $subscribeFlag = $frequency;
	    &HNMsg("found it") if $debug;
	}
    }
    if ($subscribeFlag) {
	return ($aSubscriber, $subscribeFlag, $frequency, "Direct");
    }
    else { 
	return  ('', '', '', "Not Specified"); 
    }
}



sub replaceSubscription {
    my ($dir, $userid, $newfreq) = @_;
    # Replace subscription for $userid in $dir with $newfreq.
    # $userid might be an email address.
    # If $newfreq is null, delete subscription.
    # This always writes a new file, even if there is no change.
    # It also doesn't check whether the user is auto-subscribed because
    # of being the author, rather than explicitly subscribed.
    local $debug = 0;

    my $listfile = "$hndocs$dir/list";

    $userid = &useridOf($userid) || $userid;
    return if !$userid;
    &HNMsg("userid: $userid") if $debug;

    if (! -d "$hndocs$dir") {
	# Should use create_dir, but mkdir is OK for now.
	mkdir("$hndocs$dir", "$hndirmode"); 
	chmod("$hndirmode", "$hndocs$dir");  # Redundant?
    }
    &HNMsg("listfile: $listfile") if $debug;

    require "lock.pl";
    &lock($listfile);
    open (ML, $listfile);  # might not exist.

    my $lastFrequency;
    open (MLN, ">$listfile-new") ||
	&HNError("Unable to open for writing: $listfile-new");
    while (<ML>) {
	my ($aSubscriber, $frequency) = split(':', $_);
	# Subscriber may not be a member.
	$aSubscriber =~ s|[ \n]||g;
	$aSubscriber = &useridOf($aSubscriber) || $aSubscriber;
	next if !$aSubscriber;
	$frequency =~ s|[ \n]||g;
	if ($aSubscriber eq $userid) {
	    # Don't write this line, but remember the freq for later.
	    $lastFrequency = $frequency;
	}
	else {
	    print MLN "$aSubscriber:$frequency\n"
		|| &HNError("Unable to print to $listfile-new");
	}

    }

    # Always append to the end.
    print MLN "$userid:$newfreq\n" if $newfreq;

    close(MLN);
    close (ML);
    &moveItSafely($listfile, "$listfile-old", "$listfile-new");
    &unlock($listfile);

    # Only log a change in subscription
    if ($lastFrequency ne $newfreq) {
	my $log_freq = $newfreq || 'Unspecified';
	require 'log-lib.pl';
	&logTransaction("$log_freq $userid", $dir);
    }

    &HNMsg("finished replace subscribe") if $debug;
}



sub buildList {
    my ($URL) = @_;
    # Build and return mailing list for the URL.
    # If the forum is an All-Members forum, use the all-members list too.

    my ($responses, $base, $nextUpFile);
    local($debug) = 0;

    my %list = ();

    &HNMsg("build list for $URL") if $debug;
    my %urc = &parseURCFile("$hndocs$URL,urc");
    $base = $urc{BaseURL};
    &HNMsg("base is $base") if $debug;

    my $done = 0;

    # Climb the hierarchy adding to the list at each level.
    while (!$done) {
	# Open the mailing list for this URL, if any.
	$responses = $urc{Responses};
	&HNMsg("list file: $responses/list") if $debug;
        %list = &readList("$hndocs$responses/list", %list);

	# Add the author of this message, if not already specified.
	my $from = $urc{'From'};
	$from = &useridOf($from);
	my $fromMember = &getMember($from);
	
	if (&useridOK($from) && !$list{$from}) {
	    $list{$from} = "Immediate";
	    &HNMsg ("Added $from") if $debug;
	}
	&printAssoc ("List is", %list) if $debug; 
	if ($base eq $URL) {
	    $done = 1;
	}

	$URL = $urc{UpURL};
	if (!$URL) {
	    $done = 1;
	}
	# Remove the $get prefix from the URL.
	$nextUpFile = $URL;
	$nextUpFile =~ s,^.*/get(\.cgi)?/,/,;

	# print "NextUpFile: '$nextUpFile'\n";
	%urc = &parseURCFile("$hndocs$nextUpFile,urc");
	&printAssoc("URC of $nextUpFile", %urc) if $debug;
	# print "URL = $URL\n\n";
    }

    # Add all global subscribers, if forum is Subscribe: all.
    my $hnbase = &url2hnpath($base);
    %urc = &parseURCFile("$hndocs$hnbase,urc");
    if (($urc{Subscribe} || $allMemberForums) eq 'all') {
	# Subscribe all to this all-member forum.
	&HNMsg("Adding all global subscribers") if $debug;   
	%list = &readList("$hndocs/list", %list);
    }

    # Return the list at this outer level.
    &printAssoc("Returning List", %list) if $debug;
    #print "What's going on!!!\n\n\n";
    #&flush(STDOUT);
    return %list;
}


sub readList {
  my ($filename, %list) = @_;
  # Read list in file and add to %list.

  my ($Subscriber, $frequency);
  local $debug = 0;

  if (open (ML, "<$filename")) {
    while (<ML>) {
      chop ($_);		# Trailing newline
      &HNMsg ("Next: '$_'") if $debug;
      ($Subscriber, $frequency) = split(':', $_);
      $Subscriber = &useridOf($Subscriber);
      # Add an entry for this address if there is not one already.
      if (&useridOK($Subscriber) && !$list{$Subscriber}) {
	$frequency =~ s,[ \n],,g;
	if (!$frequency) {
	  # If it is empty, assume it is Immediate.
	  $frequency = "Immediate";
	}
	$list{$Subscriber} = $frequency;
      }
    }
    close (ML);
  }
  return %list;
}


sub filterList {
    my ($frequency, %list) = @_;
    # Filter the list by returning a new list of those addresses 
    # that have the specified options.  Ignore user preferences for now.
    # Only OK addresses are returned.
    # If a subscribed member has Unsubscribed globally, that overrides.

    local $debug = 0;
    my @subscribers = keys %list;
    &HNMsg("ProcessID: $$") if $debug;
    &HNMsg("Frequency: $frequency  List: @subscribers\n") if $debug;
    &HNMsg("What is the problem here!!!!!!!!\n") if $debug;
    #&flush(STDOUT);

    my @newlist = ();
    my %seen_list = ();

    foreach $subscriber (@subscribers) {
      my ($subscribeFrequency) = $list{$subscriber};
      &HNMsg("What is the problem here!!!!!!!! Looping... \n") if $debug;
      #print "Output is not happening";
      #&flush(STDOUT);

      my $email;
      if ($frequency eq $subscribeFrequency) {
	  my (%member) = &getMember($subscriber);
	  if (&addressOK($subscriber)) {
	      $email = $subscriber;
	  }
	  else {
	      # Convert back to email address.
	      $email =  $member{'Email'};
	  }
	  if ($seen_list{$email}) {
	      $email = '';  # Already delt with.
	  }
	  else {
	      $seen_list{$email} = 1 if $email;
	  }

	  if ($member{Subscribe} eq 'none') {
	      # Global unsubscribe.
	      $email = '';
	  }
	  if ($email) {
	      &HNMsg("Adding: $email") if $debug;
	      push (@newlist, $email);
	  }
      }
    }
    &HNMsg("Returning from filterList: @newlist") if $debug;
    #&flush(STDOUT);
    return @newlist;
}

1;

